# Garvan Institute Peter Wills Bioinformatic Centre 
#
# Author Kevin Ying

import numpy
import sys
#import rpy2.robjects as robjects
#import rpy2.robjects.numpy2ri

DEBUG = 0

########################################################
# Constants
#
# Column indices of a 2D array corresponding to the columns of a GCT
# For info on the GCT file format see
# http://www.broadinstitute.org/cancer/software/genepattern/tutorial/gp_fileformats.html#gct

# Column indices for the header
# row 1
GCT_HEAD_NROWS_ROW_IDX = 1
GCT_HEAD_NROWS_COL_IDX = 0
GCT_HEAD_NSAMP_ROW_IDX = 1
GCT_HEAD_NSAMP_COL_IDX = 1

# Column indices For data
# rows 4 onwards
GCT_DATA_NAME_IDX = 0 # unique row identifier, usually probe ID
GCT_DATA_DESC_IDX = 1 # Description
GCT_DATA_SAMP_IDX = 2 # The *start* of the sample columns 
########################################################

########################################################
# GCT files
# For more details:
# http://www.broadinstitute.org/cancer/software/genepattern/tutorial/gp_fileformats.html#gct
# eg. of a GCT file, the first three lines is the header 
#
# #1.2
# 1000	130
# Name	Description	Sample1	Sample2	Sample3
# 123_a	AGRN: agrin 	15.13	123.123	123.4	
# 124_b	ESPN: espin 	16.13	124.123	124.5	
#
# 
# The GCTFile class
class GCTFile:
	def __init__(self, header, gctData):
		self.header = header
		self.gctData = gctData

	def nSamples(self):
		return self.header[GCT_HEAD_NSAMP_ROW_IDX][GCT_HEAD_NSAMP_COL_IDX]

	def nRows(self):
		return self.header[GCT_HEAD_NROWS_ROW_IDX][GCT_HEAD_NROWS_COL_IDX]

	########################################################
	# printGCT
	#
	# given an output file name, the gct header and gctlist data
	# print to the output file
	# 
	# args: output file name, GCT header, GCT datalist
	# output: 0 if successful
	########################################################
	def printGCT(self, outFile):
		# Open output file handle
		outf = open(outFile, 'w')

		# Print header first
		for i in self.header:
			print >>outf, "\t".join(i)

		# Loop through gct and print
		for i in range(len(self.gctData)):
			print >>outf, "\t".join(self.gctData[i])

		# Close output file handle
		outf.close() 
		return 0
########################################################
# readGCT
# read a gct file
#
# returns a 2D array of the file header and gct data
#
# args: location of gct file
# ouput: (headers, gctlist)
########################################################
def readGCT(file):
	# Open and save gct file
	originalGCTData = []
	f = open(file, 'rU') # "rU" allows for reading from unix, windows and mac formats
	
	# parse GCT file data and headers
	version = f.next().strip().split("\t")
	nRowSample = f.next().strip().split("\t")
	colnames = f.next().strip().split("\t")
	headers = [version, nRowSample, colnames]

	headerNumRows = int(nRowSample[GCT_HEAD_NROWS_COL_IDX]) # Whats the header say the num of rows are
	headerNumSamp = int(nRowSample[GCT_HEAD_NSAMP_COL_IDX]) # Whats the header say the num of samples are 

	colCount = headerNumSamp + GCT_DATA_SAMP_IDX # Num of columns each row of data should have
	
	for l in f:
		line = l.strip().split("\t")
		originalGCTData.append(line)
	f.close()

	###########################################################
	# Check the header values
	# num rows and num samples match the actual numbers read
	###########################################################

	actualNumRows = len(originalGCTData)
	actualNumColNames = len(colnames) # number of cols in row 3

	# Row number
	if actualNumRows != headerNumRows:
		sys.stderr.write("Warning: header value: " + str(headerNumRows) + " does not match actual no. of rows " + str(actualNumRows) + ", check the help file for more details." + '\n')

	if actualNumColNames != headerNumSamp + GCT_DATA_SAMP_IDX: # Check there is the right amount of column names
		sys.stderr.write("Warning: Column names (row 3 of the GCT) may be malformed, check the help file for more details.\n"); 
	return GCTFile(headers, originalGCTData)
	
	
########################################################

########################################################
# GCT row functions
# 
# The following is a set of functions that can be applied to each row of a GCT
########################################################
# getExprVariable
#
# args: Expr values 
# output: variable
########################################################
def getExprVariance(row):
	vals = numpy.array(row[GCT_DATA_SAMP_IDX : ])
	
	try:
		nvalues = map(float,vals) # convert strs to float
	except ValueError:
		# Handle missing or malformed values
		valid_idx = numpy.array(map(lambda x: x != '' and x != 'na' and x != 'NA', vals)) 
		vals_valid = vals[valid_idx]
		try:
			nvalues = map(float,vals_valid) # convert strs to float
		except ValueError:
			sys.stderr.write("Error: There is a value the module cannot read as a number: at row name:" + row[GCT_DATA_NAME_IDX] + '\n' + str(vals) + '\n')

	if (DEBUG):
		print ("DEBUG: getExprVariance: nvalues: " + str(nvalues))
	return numpy.var(nvalues)


########################################################
# getAvgExprLevel
#
# args: Expression values
# output: Avg expression level
########################################################
def getAvgExprLevel(row):
	vals = numpy.array(row[GCT_DATA_SAMP_IDX : ]) 

	try:
		nvalues = map(float,vals) # convert strs to float
	except ValueError:
		# Handle missing or malformed values
		valid_idx = numpy.array(map(lambda x: x != '' and x != 'na' and x != 'NA', vals)) 
		vals_valid = vals[valid_idx]
		try:
			nvalues = map(float,vals_valid) # convert strs to float
		except ValueError:
			sys.stderr.write("Error: There is a value the module cannot read as a number: at row name:" + row[GCT_DATA_NAME_IDX] + '\n' + str(vals) + '\n')

	if (DEBUG):
		print ("DEBUG: getAvgExprLevel: numpymean: " + str(numpy.mean(nvalues)))
		print ("DEBUG: getAvgExprLevel: numpyvalues: " + str(nvalues))
	return numpy.mean(nvalues)

########################################################
# getExprSum
#
# args: Expression values
# output: Avg expression level
########################################################
def getExprSum(row):
	vals = numpy.array(row[GCT_DATA_SAMP_IDX : ]) 

	try:
		nvalues = map(float,vals) # convert strs to float
	except ValueError:
		# Handle missing or malformed values
		valid_idx = numpy.array(map(lambda x: x != '' and x != 'na' and x != 'NA', vals)) 
		vals_valid = vals[valid_idx]

		# Try and convert now
		try:
			nvalues = map(float,vals_valid) # convert strs to float
		except ValueError:
			sys.stderr.write("Error: There is a value the module cannot read as a number: at row name:" + row[GCT_DATA_NAME_IDX] + '\n' + str(vals) + '\n')

	if (DEBUG):
		print ("DEBUG: getExprSum: sum: " + str(numpy.sum(nvalues)))
		print ("DEBUG: getAvgExprLevel: numpyvalues: " + str(nvalues))
		
	return numpy.sum(nvalues)


########################################################
# getGeneSymbol
#
# From a description field of a GCT file
# Grab the gene symbol
# 
# The field must be of this format:
# eg.
# {geneSymbol}
# {geneSymbol}: rest of string
# {geneSymbol}:: rest of string
#
# args: Description string
# output: geneSymbol
########################################################
def getGeneSymbol(row):
	descStr	= row[GCT_DATA_DESC_IDX]
	return descStr.split(':')[0].strip()
